/* ****************************************************************************************************** */
/*   VHFDVXBootFlash.cmd			LINKER  SCRIPT                                                        */
/*                                                                                                        */
/*                                                                                                        */
/*   The Linker Script defines how the code and data emitted by the GNU C compiler and assembler are  	  */
/*   to be loaded into memory (code goes into FLASH, variables go into RAM).              			  	  */
/*                                                                                                        */
/*   Any symbols defined in the Linker Script are automatically global and available to the rest of the   */
/*   program.                                                                                             */
/*                                                                                                        */
/*   To force the linker to use this LINKER SCRIPT, just add the -T demo_at91sam7_blink_flash.cmd         */
/*   directive to the linker flags in the makefile. For example,                                          */
/*                                                                                                        */
/*   			LFLAGS  =  -Map main.map -nostartfiles -T VHFDVXBootFlash.cmd                               */
/*                                                                                                        */
/*                                                                                                        */
/*   The order that the object files are listed in the makefile determines what .text section is          */
/*   placed first.                                                                                        */
/*                                                                                                        */
/*   For example:  $(LD) $(LFLAGS) -o main.out  crt.o main.o lowlevelinit.o                               */
/*                                                                                                        */
/*   			   crt.o is first in the list of objects, so it will be placed at address 0x00000000      */
/*                                                                                                        */
/*                                                                                                        */
/*   The top of the stack (_stack_end) is (last_byte_of_ram +1) - 4  				                      */
/*                                                                                                        */
/*   Therefore:   _stack_end = (0x00020FFFF + 1) - 4  =  0x00021000 - 4  =  0x0020FFFC                    */
/*                                                                                                        */
/*   Note that this symbol (_stack_end) is automatically GLOBAL and will be used by the crt.s             */
/*   startup assembler routine to specify all stacks for the various ARM modes                            */
/*                                                                                                        */
/* ==========================================================================*/
/*                   MEMORY MAP                                              */
/*      .-------->|---------------------------------|0x00210000              */
/*      .         |                                 |0x0020FFFC  _stack_end  */
/*      .         |    UDF Stack  16 bytes          |                        */
/*      .         |                                 |                        */
/*      .         |---------------------------------|0x0020FFEC              */
/*      .         |                                 |                        */
/*      .         |    ABT Stack  16 bytes          |                        */
/*      .         |                                 |                        */
/*      .         |---------------------------------|0x0020FFDC              */
/*      .         |                                 |                        */
/*      .         |                                 |                        */
/*      .         |    FIQ Stack  128 bytes         |                        */
/*      .         |                                 |                        */
/*      .         |                                 |                        */
/*     RAM        |---------------------------------|0x0020FF5C              */
/*      .         |                                 |                        */
/*      .         |                                 |                        */
/*      .         |    IRQ Stack  128 bytes         |                        */
/*      .         |                                 |                        */
/*      .         |                                 |                        */
/*      .         |---------------------------------|0x0020FEDC              */
/*      .         |                                 |                        */
/*      .         |    SVC Stack  16 bytes          |                        */
/*      .         |                                 |                        */
/*      .         |---------------------------------|0x0020FECC              */
/*      .         |                                 |                        */
/*      .         |     stack area for user program |                        */
/*      .         |                                 |                        */
/*      .         |                                 |                        */
/*      .         |                                 |                        */
/*      .         |          free ram               |                        */
/*      .         |                                 |                        */
/*      .         |.................................|  _bss_end              */
/*      .         |                                 |                        */
/*      .         |  .bss   uninitialized variables |                        */
/*      .         |.................................|  _bss_start, _edata    */
/*      .         |                                 |                        */
/*      .         |  .data  initialized variables   |                        */
/*      .         |                                 |                        */
/*      .-------->|_________________________________|0x00200000              */
/*                                                                           */
/*                                                                           */
/*      .--------> |---------------------------------|                       */
/*      .          |                                 |0x0013FFFF             */
/*      .          |                                 |                       */
/*      .          |    User  Flash Blocks           |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*    Flash        |                                 |                       */
/*      .          |.................................|0x00104040             */
/*      .          |   User Vectors and tables       |0x0010403F             */
/*      .          |   to be copied to RAM before    |                       */
/*      .          |  executing user code            |                       */
/*      .          |.................................|0x00104000             */
/*      .          |                                 |0x00103FFF             */
/*      .          | Protected BootLoader Block      |                       */
/*      .          |                                 |                       */
/*      .--------->|---------------------------------|0x00100000             */
/*                                                                           */
/*                                                                           */
/*      .--------> |---------------------------------|                       */
/*      .          |                                 |0x0003FFFF (0x0000FFFF)*/
/*      .          |                                 |           after remap)*/
/*      .          |    Flash or Ram after remap     |                       */
/*      .          |                                 |                       */
/*    Flash        |                                 |                       */
/*     or          |                                 |                       */
/*  (Ram after     |                                 |                       */
/*    remap)       |                                 |                       */
/*      .          |                                 |                       */
/*      .          |.................................|0x00000040             */
/*      .          | Vectors and jump tables         |0x0000003F             */
/*      .          | Copied from user flash before   |                       */
/*      .          |  jumping to user code           |                       */
/*      .--------->|---------------------------------|0x00000000             */
/* ==========================================================================*/


MEMORY
{
  CODE (rx)  : ORIGIN = 0x00000000, LENGTH = 0x00004000	/* use first 16K AT91SAM7256 flash block */
  BOOT (rw)  : ORIGIN = 0x00200000, LENGTH = 0x00004000	/* bootloader block copied and run from RAM */
  DATA (rw)  : ORIGIN = 0x00204000, LENGTH = 0x0000F000
  STACK (rw) : ORIGIN = 0x00210000, LENGTH = 0x00000000
}

/* now define the output sections  */
SECTIONS 
{
	. = 0x0;
	_text = . ;
	
	.text :								/* collect all sections that should go into FLASH after startup  */ 
	{
		*(.startup)                     /* Startup section must begin at (0x00000000)
		*(.text)						/* all .text sections (code)  */
		*(.rodata)						/* all .rodata sections (constants, strings, etc.)  */
		*(.rodata*)						/* all .rodata* sections (constants, strings, etc.)  */
		*(.glue_7)						/* all .glue_7 sections  (no idea what these are) */
		*(.glue_7t)						/* all .glue_7t sections (no idea what these are) */
		_etext = .;						/* define a global symbol _etext just after the last code byte */
	} >CODE 							/* put all the above into FLASH */

	.data :								/* collect all initialized .data sections that go into RAM  */ 
	{
		_data = .;						/* create a global symbol marking the start of the .data section  */
		*(.data)						/* all .data sections  */
        SORT(CONSTRUCTORS)
		_edata = .;						/* define a global symbol marking the end of the .data section  */
	} >DATA AT >CODE          			/* put all the above into RAM (but load the LMA initializer copy into FLASH)  */

	.bss :								/* collect all uninitialized .bss sections that go into RAM  */
	{
		_bss_start = .;					/* define a global symbol marking the start of the .bss section */
		*(.bss)							/* all .bss sections  */
	} >DATA								/* put all the above in RAM (it will be cleared in the startup code */

	. = ALIGN(4);						/* advance location counter to the next 32-bit boundary */
	_bss_end = . ;						/* define a global symbol marking the end of the .bss section */
   .int_data :  						/* define stack area */
   { 
   		*(.internal_ram_top) 
   }> STACK 
}